<?php
/**
 * @file
 * Contains \Drupal\critique_and_review\Plugin\Block\CritiqueAndReviewBlock.
 */

namespace Drupal\critique_and_review\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Plugin\views\filter\Access;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Provides a 'critique_and_review' block.
 *
 * @Block(
 *   id = "critique_and_review_block",
 *   admin_label = @Translation("Critique And Review Block"),
 *   category = @Translation("Custom Critique And Review Block")
 * )
 */
class CritiqueAndReviewBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Get current node
    $node = \Drupal::routeMatch()->getParameter('node');

    if(currentUserIsNodeCreator($node)) {
      $url = Url::fromRoute('entity.node.edit_form', array('node' => $node->id()));
      $project_link = Link::fromTextAndUrl(t('Edit'), $url);
      $project_link = $project_link->toRenderable();
      $project_link['#attributes'] = array('class' => array('button', 'button-action', 'button--primary', 'button--small'));
      $build['markup'] = [
        '#type' => 'item',
        '#markup' => "You have reviews. Edit to view them.",
        '#attributes' => ['id' => 'nicey-template-review-form-bottom', 'class' => ['nicey-template-review-form-wrapper']],
        '#cache' =>   [
          'contexts' => ['user'],
          'tags' => [],
         'max-age' => 0,
        ]
      ];
      $build['link'] = $project_link;
      $build['link']['#cache'] = [
          'contexts' => ['user'],
          'tags' => [],
          'max-age' => 0,
      ];
    } else {
       $build = [
        '#type' => 'markup',
        '#attributes' => ['id' => 'nicey-template-review-form', 'class' => ['nicey-template-review-form-wrapper']],
        'form' => \Drupal::formBuilder()->getForm('Drupal\critique_and_review\Form\TemplateCritiqueAndReviewForm'),
        '#cache' =>   [
          'contexts' => ['user'],
          'tags' => [],
          'max-age' => 0,
      ]
     ];
    }
    return $build;
   }
}
